# coding: utf-8

"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class BillingEntityCreate(BaseModel):
    """
    BillingEntityCreate
    """ # noqa: E501
    code: Annotated[str, Field(min_length=1, strict=True, max_length=100)]
    name: Annotated[str, Field(min_length=1, strict=True, max_length=255)]
    legal_name: Optional[Annotated[str, Field(strict=True, max_length=255)]] = None
    address_line1: Optional[StrictStr] = None
    address_line2: Optional[StrictStr] = None
    city: Optional[StrictStr] = None
    state: Optional[StrictStr] = None
    country: Optional[Annotated[str, Field(min_length=2, strict=True, max_length=2)]] = None
    zip_code: Optional[StrictStr] = None
    tax_id: Optional[StrictStr] = None
    email: Optional[StrictStr] = None
    currency: Optional[Annotated[str, Field(min_length=3, strict=True, max_length=3)]] = 'USD'
    timezone: Optional[Annotated[str, Field(strict=True, max_length=50)]] = 'UTC'
    document_locale: Optional[Annotated[str, Field(strict=True, max_length=10)]] = 'en'
    invoice_prefix: Optional[Annotated[str, Field(strict=True, max_length=20)]] = None
    next_invoice_number: Optional[Annotated[int, Field(strict=True, ge=1)]] = 1
    invoice_grace_period: Optional[Annotated[int, Field(strict=True, ge=0)]] = 0
    net_payment_term: Optional[Annotated[int, Field(strict=True, ge=0)]] = 30
    invoice_footer: Optional[Annotated[str, Field(strict=True, max_length=1024)]] = None
    is_default: Optional[StrictBool] = False
    __properties: ClassVar[List[str]] = ["code", "name", "legal_name", "address_line1", "address_line2", "city", "state", "country", "zip_code", "tax_id", "email", "currency", "timezone", "document_locale", "invoice_prefix", "next_invoice_number", "invoice_grace_period", "net_payment_term", "invoice_footer", "is_default"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of BillingEntityCreate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if legal_name (nullable) is None
        # and model_fields_set contains the field
        if self.legal_name is None and "legal_name" in self.model_fields_set:
            _dict['legal_name'] = None

        # set to None if address_line1 (nullable) is None
        # and model_fields_set contains the field
        if self.address_line1 is None and "address_line1" in self.model_fields_set:
            _dict['address_line1'] = None

        # set to None if address_line2 (nullable) is None
        # and model_fields_set contains the field
        if self.address_line2 is None and "address_line2" in self.model_fields_set:
            _dict['address_line2'] = None

        # set to None if city (nullable) is None
        # and model_fields_set contains the field
        if self.city is None and "city" in self.model_fields_set:
            _dict['city'] = None

        # set to None if state (nullable) is None
        # and model_fields_set contains the field
        if self.state is None and "state" in self.model_fields_set:
            _dict['state'] = None

        # set to None if country (nullable) is None
        # and model_fields_set contains the field
        if self.country is None and "country" in self.model_fields_set:
            _dict['country'] = None

        # set to None if zip_code (nullable) is None
        # and model_fields_set contains the field
        if self.zip_code is None and "zip_code" in self.model_fields_set:
            _dict['zip_code'] = None

        # set to None if tax_id (nullable) is None
        # and model_fields_set contains the field
        if self.tax_id is None and "tax_id" in self.model_fields_set:
            _dict['tax_id'] = None

        # set to None if email (nullable) is None
        # and model_fields_set contains the field
        if self.email is None and "email" in self.model_fields_set:
            _dict['email'] = None

        # set to None if invoice_prefix (nullable) is None
        # and model_fields_set contains the field
        if self.invoice_prefix is None and "invoice_prefix" in self.model_fields_set:
            _dict['invoice_prefix'] = None

        # set to None if invoice_footer (nullable) is None
        # and model_fields_set contains the field
        if self.invoice_footer is None and "invoice_footer" in self.model_fields_set:
            _dict['invoice_footer'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BillingEntityCreate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "code": obj.get("code"),
            "name": obj.get("name"),
            "legal_name": obj.get("legal_name"),
            "address_line1": obj.get("address_line1"),
            "address_line2": obj.get("address_line2"),
            "city": obj.get("city"),
            "state": obj.get("state"),
            "country": obj.get("country"),
            "zip_code": obj.get("zip_code"),
            "tax_id": obj.get("tax_id"),
            "email": obj.get("email"),
            "currency": obj.get("currency") if obj.get("currency") is not None else 'USD',
            "timezone": obj.get("timezone") if obj.get("timezone") is not None else 'UTC',
            "document_locale": obj.get("document_locale") if obj.get("document_locale") is not None else 'en',
            "invoice_prefix": obj.get("invoice_prefix"),
            "next_invoice_number": obj.get("next_invoice_number") if obj.get("next_invoice_number") is not None else 1,
            "invoice_grace_period": obj.get("invoice_grace_period") if obj.get("invoice_grace_period") is not None else 0,
            "net_payment_term": obj.get("net_payment_term") if obj.get("net_payment_term") is not None else 30,
            "invoice_footer": obj.get("invoice_footer"),
            "is_default": obj.get("is_default") if obj.get("is_default") is not None else False
        })
        return _obj





"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from datetime import datetime
from pydantic import Field, StrictStr
from typing import List, Optional
from typing_extensions import Annotated
from uuid import UUID
from bxb.models.estimate_fees_request import EstimateFeesRequest
from bxb.models.estimate_fees_response import EstimateFeesResponse
from bxb.models.event_batch_create import EventBatchCreate
from bxb.models.event_batch_response import EventBatchResponse
from bxb.models.event_create import EventCreate
from bxb.models.event_reprocess_response import EventReprocessResponse
from bxb.models.event_response import EventResponse
from bxb.models.event_volume_response import EventVolumeResponse

from bxb.api_client import ApiClient, RequestSerialized
from bxb.api_response import ApiResponse
from bxb.rest import RESTResponseType


class EventsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def create_event_v1_events_post(
        self,
        event_create: EventCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> EventResponse:
        """Ingest event

        Ingest a single event.  If an event with the same transaction_id already exists, returns the existing event. This provides idempotent event ingestion.  The ``Idempotency-Key`` header adds an additional layer of deduplication on top of the existing ``transaction_id`` check.

        :param event_create: (required)
        :type event_create: EventCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_event_v1_events_post_serialize(
            event_create=event_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "EventResponse",
            '401': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_event_v1_events_post_with_http_info(
        self,
        event_create: EventCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[EventResponse]:
        """Ingest event

        Ingest a single event.  If an event with the same transaction_id already exists, returns the existing event. This provides idempotent event ingestion.  The ``Idempotency-Key`` header adds an additional layer of deduplication on top of the existing ``transaction_id`` check.

        :param event_create: (required)
        :type event_create: EventCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_event_v1_events_post_serialize(
            event_create=event_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "EventResponse",
            '401': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_event_v1_events_post_without_preload_content(
        self,
        event_create: EventCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Ingest event

        Ingest a single event.  If an event with the same transaction_id already exists, returns the existing event. This provides idempotent event ingestion.  The ``Idempotency-Key`` header adds an additional layer of deduplication on top of the existing ``transaction_id`` check.

        :param event_create: (required)
        :type event_create: EventCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_event_v1_events_post_serialize(
            event_create=event_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "EventResponse",
            '401': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_event_v1_events_post_serialize(
        self,
        event_create,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if event_create is not None:
            _body_params = event_create


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/v1/events/',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def create_events_batch_v1_events_batch_post(
        self,
        event_batch_create: EventBatchCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> EventBatchResponse:
        """Ingest event batch

        Ingest a batch of events (up to 100).  Duplicate transaction_ids are handled gracefully - existing events are returned without error. The response includes counts of newly ingested vs duplicate events.

        :param event_batch_create: (required)
        :type event_batch_create: EventBatchCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_events_batch_v1_events_batch_post_serialize(
            event_batch_create=event_batch_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "EventBatchResponse",
            '401': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_events_batch_v1_events_batch_post_with_http_info(
        self,
        event_batch_create: EventBatchCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[EventBatchResponse]:
        """Ingest event batch

        Ingest a batch of events (up to 100).  Duplicate transaction_ids are handled gracefully - existing events are returned without error. The response includes counts of newly ingested vs duplicate events.

        :param event_batch_create: (required)
        :type event_batch_create: EventBatchCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_events_batch_v1_events_batch_post_serialize(
            event_batch_create=event_batch_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "EventBatchResponse",
            '401': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_events_batch_v1_events_batch_post_without_preload_content(
        self,
        event_batch_create: EventBatchCreate,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Ingest event batch

        Ingest a batch of events (up to 100).  Duplicate transaction_ids are handled gracefully - existing events are returned without error. The response includes counts of newly ingested vs duplicate events.

        :param event_batch_create: (required)
        :type event_batch_create: EventBatchCreate
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_events_batch_v1_events_batch_post_serialize(
            event_batch_create=event_batch_create,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "EventBatchResponse",
            '401': None,
            '422': None,
            '429': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_events_batch_v1_events_batch_post_serialize(
        self,
        event_batch_create,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if event_batch_create is not None:
            _body_params = event_batch_create


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/v1/events/batch',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def estimate_fees_v1_events_estimate_fees_post(
        self,
        estimate_fees_request: EstimateFeesRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> EstimateFeesResponse:
        """Estimate fees for a hypothetical event

        Estimate fees from a hypothetical event.  Aggregates current-period usage for the given metric, adds the hypothetical event's contribution, and applies the charge model to return the estimated fee amount.

        :param estimate_fees_request: (required)
        :type estimate_fees_request: EstimateFeesRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._estimate_fees_v1_events_estimate_fees_post_serialize(
            estimate_fees_request=estimate_fees_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EstimateFeesResponse",
            '401': None,
            '404': None,
            '422': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def estimate_fees_v1_events_estimate_fees_post_with_http_info(
        self,
        estimate_fees_request: EstimateFeesRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[EstimateFeesResponse]:
        """Estimate fees for a hypothetical event

        Estimate fees from a hypothetical event.  Aggregates current-period usage for the given metric, adds the hypothetical event's contribution, and applies the charge model to return the estimated fee amount.

        :param estimate_fees_request: (required)
        :type estimate_fees_request: EstimateFeesRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._estimate_fees_v1_events_estimate_fees_post_serialize(
            estimate_fees_request=estimate_fees_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EstimateFeesResponse",
            '401': None,
            '404': None,
            '422': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def estimate_fees_v1_events_estimate_fees_post_without_preload_content(
        self,
        estimate_fees_request: EstimateFeesRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Estimate fees for a hypothetical event

        Estimate fees from a hypothetical event.  Aggregates current-period usage for the given metric, adds the hypothetical event's contribution, and applies the charge model to return the estimated fee amount.

        :param estimate_fees_request: (required)
        :type estimate_fees_request: EstimateFeesRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._estimate_fees_v1_events_estimate_fees_post_serialize(
            estimate_fees_request=estimate_fees_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EstimateFeesResponse",
            '401': None,
            '404': None,
            '422': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _estimate_fees_v1_events_estimate_fees_post_serialize(
        self,
        estimate_fees_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if estimate_fees_request is not None:
            _body_params = estimate_fees_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/v1/events/estimate_fees',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_event_v1_events_event_id_get(
        self,
        event_id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> EventResponse:
        """Get event

        Get an event by ID.

        :param event_id: (required)
        :type event_id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_event_v1_events_event_id_get_serialize(
            event_id=event_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EventResponse",
            '401': None,
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_event_v1_events_event_id_get_with_http_info(
        self,
        event_id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[EventResponse]:
        """Get event

        Get an event by ID.

        :param event_id: (required)
        :type event_id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_event_v1_events_event_id_get_serialize(
            event_id=event_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EventResponse",
            '401': None,
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_event_v1_events_event_id_get_without_preload_content(
        self,
        event_id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get event

        Get an event by ID.

        :param event_id: (required)
        :type event_id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_event_v1_events_event_id_get_serialize(
            event_id=event_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EventResponse",
            '401': None,
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_event_v1_events_event_id_get_serialize(
        self,
        event_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if event_id is not None:
            _path_params['event_id'] = event_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v1/events/{event_id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_event_volume_v1_events_volume_get(
        self,
        from_timestamp: Optional[datetime] = None,
        to_timestamp: Optional[datetime] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> EventVolumeResponse:
        """Get event volume (events per hour)

        Get hourly event volume data for charting.

        :param from_timestamp:
        :type from_timestamp: datetime
        :param to_timestamp:
        :type to_timestamp: datetime
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_event_volume_v1_events_volume_get_serialize(
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EventVolumeResponse",
            '401': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_event_volume_v1_events_volume_get_with_http_info(
        self,
        from_timestamp: Optional[datetime] = None,
        to_timestamp: Optional[datetime] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[EventVolumeResponse]:
        """Get event volume (events per hour)

        Get hourly event volume data for charting.

        :param from_timestamp:
        :type from_timestamp: datetime
        :param to_timestamp:
        :type to_timestamp: datetime
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_event_volume_v1_events_volume_get_serialize(
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EventVolumeResponse",
            '401': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_event_volume_v1_events_volume_get_without_preload_content(
        self,
        from_timestamp: Optional[datetime] = None,
        to_timestamp: Optional[datetime] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get event volume (events per hour)

        Get hourly event volume data for charting.

        :param from_timestamp:
        :type from_timestamp: datetime
        :param to_timestamp:
        :type to_timestamp: datetime
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_event_volume_v1_events_volume_get_serialize(
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EventVolumeResponse",
            '401': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_event_volume_v1_events_volume_get_serialize(
        self,
        from_timestamp,
        to_timestamp,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if from_timestamp is not None:
            if isinstance(from_timestamp, datetime):
                _query_params.append(
                    (
                        'from_timestamp',
                        from_timestamp.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('from_timestamp', from_timestamp))
            
        if to_timestamp is not None:
            if isinstance(to_timestamp, datetime):
                _query_params.append(
                    (
                        'to_timestamp',
                        to_timestamp.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('to_timestamp', to_timestamp))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v1/events/volume',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_events_v1_events_get(
        self,
        skip: Optional[Annotated[int, Field(strict=True, ge=0)]] = None,
        limit: Optional[Annotated[int, Field(le=1000, strict=True, ge=1)]] = None,
        order_by: Optional[StrictStr] = None,
        external_customer_id: Optional[StrictStr] = None,
        code: Optional[StrictStr] = None,
        from_timestamp: Optional[datetime] = None,
        to_timestamp: Optional[datetime] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[EventResponse]:
        """List events

        List events with optional filters.

        :param skip:
        :type skip: int
        :param limit:
        :type limit: int
        :param order_by:
        :type order_by: str
        :param external_customer_id:
        :type external_customer_id: str
        :param code:
        :type code: str
        :param from_timestamp:
        :type from_timestamp: datetime
        :param to_timestamp:
        :type to_timestamp: datetime
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_events_v1_events_get_serialize(
            skip=skip,
            limit=limit,
            order_by=order_by,
            external_customer_id=external_customer_id,
            code=code,
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[EventResponse]",
            '401': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_events_v1_events_get_with_http_info(
        self,
        skip: Optional[Annotated[int, Field(strict=True, ge=0)]] = None,
        limit: Optional[Annotated[int, Field(le=1000, strict=True, ge=1)]] = None,
        order_by: Optional[StrictStr] = None,
        external_customer_id: Optional[StrictStr] = None,
        code: Optional[StrictStr] = None,
        from_timestamp: Optional[datetime] = None,
        to_timestamp: Optional[datetime] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[EventResponse]]:
        """List events

        List events with optional filters.

        :param skip:
        :type skip: int
        :param limit:
        :type limit: int
        :param order_by:
        :type order_by: str
        :param external_customer_id:
        :type external_customer_id: str
        :param code:
        :type code: str
        :param from_timestamp:
        :type from_timestamp: datetime
        :param to_timestamp:
        :type to_timestamp: datetime
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_events_v1_events_get_serialize(
            skip=skip,
            limit=limit,
            order_by=order_by,
            external_customer_id=external_customer_id,
            code=code,
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[EventResponse]",
            '401': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_events_v1_events_get_without_preload_content(
        self,
        skip: Optional[Annotated[int, Field(strict=True, ge=0)]] = None,
        limit: Optional[Annotated[int, Field(le=1000, strict=True, ge=1)]] = None,
        order_by: Optional[StrictStr] = None,
        external_customer_id: Optional[StrictStr] = None,
        code: Optional[StrictStr] = None,
        from_timestamp: Optional[datetime] = None,
        to_timestamp: Optional[datetime] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List events

        List events with optional filters.

        :param skip:
        :type skip: int
        :param limit:
        :type limit: int
        :param order_by:
        :type order_by: str
        :param external_customer_id:
        :type external_customer_id: str
        :param code:
        :type code: str
        :param from_timestamp:
        :type from_timestamp: datetime
        :param to_timestamp:
        :type to_timestamp: datetime
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_events_v1_events_get_serialize(
            skip=skip,
            limit=limit,
            order_by=order_by,
            external_customer_id=external_customer_id,
            code=code,
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[EventResponse]",
            '401': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_events_v1_events_get_serialize(
        self,
        skip,
        limit,
        order_by,
        external_customer_id,
        code,
        from_timestamp,
        to_timestamp,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if skip is not None:
            
            _query_params.append(('skip', skip))
            
        if limit is not None:
            
            _query_params.append(('limit', limit))
            
        if order_by is not None:
            
            _query_params.append(('order_by', order_by))
            
        if external_customer_id is not None:
            
            _query_params.append(('external_customer_id', external_customer_id))
            
        if code is not None:
            
            _query_params.append(('code', code))
            
        if from_timestamp is not None:
            if isinstance(from_timestamp, datetime):
                _query_params.append(
                    (
                        'from_timestamp',
                        from_timestamp.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('from_timestamp', from_timestamp))
            
        if to_timestamp is not None:
            if isinstance(to_timestamp, datetime):
                _query_params.append(
                    (
                        'to_timestamp',
                        to_timestamp.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('to_timestamp', to_timestamp))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/v1/events/',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def reprocess_event_v1_events_event_id_reprocess_post(
        self,
        event_id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> EventReprocessResponse:
        """Reprocess an event

        Re-trigger usage threshold and alert checks for an existing event.  This re-enqueues background jobs (usage threshold checks and usage alert checks) for all active subscriptions belonging to the event's customer. The event data itself is not modified.

        :param event_id: (required)
        :type event_id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._reprocess_event_v1_events_event_id_reprocess_post_serialize(
            event_id=event_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EventReprocessResponse",
            '401': None,
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def reprocess_event_v1_events_event_id_reprocess_post_with_http_info(
        self,
        event_id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[EventReprocessResponse]:
        """Reprocess an event

        Re-trigger usage threshold and alert checks for an existing event.  This re-enqueues background jobs (usage threshold checks and usage alert checks) for all active subscriptions belonging to the event's customer. The event data itself is not modified.

        :param event_id: (required)
        :type event_id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._reprocess_event_v1_events_event_id_reprocess_post_serialize(
            event_id=event_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EventReprocessResponse",
            '401': None,
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def reprocess_event_v1_events_event_id_reprocess_post_without_preload_content(
        self,
        event_id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Reprocess an event

        Re-trigger usage threshold and alert checks for an existing event.  This re-enqueues background jobs (usage threshold checks and usage alert checks) for all active subscriptions belonging to the event's customer. The event data itself is not modified.

        :param event_id: (required)
        :type event_id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._reprocess_event_v1_events_event_id_reprocess_post_serialize(
            event_id=event_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "EventReprocessResponse",
            '401': None,
            '404': None,
            '422': "HTTPValidationError",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _reprocess_event_v1_events_event_id_reprocess_post_serialize(
        self,
        event_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if event_id is not None:
            _path_params['event_id'] = event_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/v1/events/{event_id}/reprocess',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List
from typing_extensions import Annotated
from bxb.models.fee_preview import FeePreview
from typing import Optional, Set
from typing_extensions import Self

class InvoicePreviewResponse(BaseModel):
    """
    Response for an invoice preview.
    """ # noqa: E501
    subtotal_cents: Annotated[str, Field(strict=True)]
    tax_amount_cents: Annotated[str, Field(strict=True)]
    coupons_amount_cents: Annotated[str, Field(strict=True)]
    prepaid_credit_amount_cents: Annotated[str, Field(strict=True)]
    total_cents: Annotated[str, Field(strict=True)]
    currency: StrictStr
    fees: List[FeePreview]
    __properties: ClassVar[List[str]] = ["subtotal_cents", "tax_amount_cents", "coupons_amount_cents", "prepaid_credit_amount_cents", "total_cents", "currency", "fees"]

    @field_validator('subtotal_cents')
    def subtotal_cents_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^(?!^[-+.]*$)[+-]?0*\d*\.?\d*$", value):
            raise ValueError(r"must validate the regular expression /^(?!^[-+.]*$)[+-]?0*\d*\.?\d*$/")
        return value

    @field_validator('tax_amount_cents')
    def tax_amount_cents_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^(?!^[-+.]*$)[+-]?0*\d*\.?\d*$", value):
            raise ValueError(r"must validate the regular expression /^(?!^[-+.]*$)[+-]?0*\d*\.?\d*$/")
        return value

    @field_validator('coupons_amount_cents')
    def coupons_amount_cents_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^(?!^[-+.]*$)[+-]?0*\d*\.?\d*$", value):
            raise ValueError(r"must validate the regular expression /^(?!^[-+.]*$)[+-]?0*\d*\.?\d*$/")
        return value

    @field_validator('prepaid_credit_amount_cents')
    def prepaid_credit_amount_cents_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^(?!^[-+.]*$)[+-]?0*\d*\.?\d*$", value):
            raise ValueError(r"must validate the regular expression /^(?!^[-+.]*$)[+-]?0*\d*\.?\d*$/")
        return value

    @field_validator('total_cents')
    def total_cents_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^(?!^[-+.]*$)[+-]?0*\d*\.?\d*$", value):
            raise ValueError(r"must validate the regular expression /^(?!^[-+.]*$)[+-]?0*\d*\.?\d*$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of InvoicePreviewResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in fees (list)
        _items = []
        if self.fees:
            for _item_fees in self.fees:
                if _item_fees:
                    _items.append(_item_fees.to_dict())
            _dict['fees'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of InvoicePreviewResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "subtotal_cents": obj.get("subtotal_cents"),
            "tax_amount_cents": obj.get("tax_amount_cents"),
            "coupons_amount_cents": obj.get("coupons_amount_cents"),
            "prepaid_credit_amount_cents": obj.get("prepaid_credit_amount_cents"),
            "total_cents": obj.get("total_cents"),
            "currency": obj.get("currency"),
            "fees": [FeePreview.from_dict(_item) for _item in obj["fees"]] if obj.get("fees") is not None else None
        })
        return _obj





# coding: utf-8

"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictBool, StrictInt
from typing import Any, ClassVar, Dict, List, Optional
from uuid import UUID
from bxb.models.billing_time import BillingTime
from bxb.models.subscription_status import SubscriptionStatus
from bxb.models.termination_action import TerminationAction
from typing import Optional, Set
from typing_extensions import Self

class SubscriptionUpdate(BaseModel):
    """
    SubscriptionUpdate
    """ # noqa: E501
    plan_id: Optional[UUID] = None
    status: Optional[SubscriptionStatus] = None
    ending_at: Optional[datetime] = None
    canceled_at: Optional[datetime] = None
    billing_time: Optional[BillingTime] = None
    trial_period_days: Optional[StrictInt] = None
    trial_ended_at: Optional[datetime] = None
    subscription_at: Optional[datetime] = None
    pay_in_advance: Optional[StrictBool] = None
    previous_plan_id: Optional[UUID] = None
    downgraded_at: Optional[datetime] = None
    on_termination_action: Optional[TerminationAction] = None
    __properties: ClassVar[List[str]] = ["plan_id", "status", "ending_at", "canceled_at", "billing_time", "trial_period_days", "trial_ended_at", "subscription_at", "pay_in_advance", "previous_plan_id", "downgraded_at", "on_termination_action"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SubscriptionUpdate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if plan_id (nullable) is None
        # and model_fields_set contains the field
        if self.plan_id is None and "plan_id" in self.model_fields_set:
            _dict['plan_id'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['status'] = None

        # set to None if ending_at (nullable) is None
        # and model_fields_set contains the field
        if self.ending_at is None and "ending_at" in self.model_fields_set:
            _dict['ending_at'] = None

        # set to None if canceled_at (nullable) is None
        # and model_fields_set contains the field
        if self.canceled_at is None and "canceled_at" in self.model_fields_set:
            _dict['canceled_at'] = None

        # set to None if billing_time (nullable) is None
        # and model_fields_set contains the field
        if self.billing_time is None and "billing_time" in self.model_fields_set:
            _dict['billing_time'] = None

        # set to None if trial_period_days (nullable) is None
        # and model_fields_set contains the field
        if self.trial_period_days is None and "trial_period_days" in self.model_fields_set:
            _dict['trial_period_days'] = None

        # set to None if trial_ended_at (nullable) is None
        # and model_fields_set contains the field
        if self.trial_ended_at is None and "trial_ended_at" in self.model_fields_set:
            _dict['trial_ended_at'] = None

        # set to None if subscription_at (nullable) is None
        # and model_fields_set contains the field
        if self.subscription_at is None and "subscription_at" in self.model_fields_set:
            _dict['subscription_at'] = None

        # set to None if pay_in_advance (nullable) is None
        # and model_fields_set contains the field
        if self.pay_in_advance is None and "pay_in_advance" in self.model_fields_set:
            _dict['pay_in_advance'] = None

        # set to None if previous_plan_id (nullable) is None
        # and model_fields_set contains the field
        if self.previous_plan_id is None and "previous_plan_id" in self.model_fields_set:
            _dict['previous_plan_id'] = None

        # set to None if downgraded_at (nullable) is None
        # and model_fields_set contains the field
        if self.downgraded_at is None and "downgraded_at" in self.model_fields_set:
            _dict['downgraded_at'] = None

        # set to None if on_termination_action (nullable) is None
        # and model_fields_set contains the field
        if self.on_termination_action is None and "on_termination_action" in self.model_fields_set:
            _dict['on_termination_action'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SubscriptionUpdate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "plan_id": obj.get("plan_id"),
            "status": obj.get("status"),
            "ending_at": obj.get("ending_at"),
            "canceled_at": obj.get("canceled_at"),
            "billing_time": obj.get("billing_time"),
            "trial_period_days": obj.get("trial_period_days"),
            "trial_ended_at": obj.get("trial_ended_at"),
            "subscription_at": obj.get("subscription_at"),
            "pay_in_advance": obj.get("pay_in_advance"),
            "previous_plan_id": obj.get("previous_plan_id"),
            "downgraded_at": obj.get("downgraded_at"),
            "on_termination_action": obj.get("on_termination_action")
        })
        return _obj





# flake8: noqa

# import apis into api package
from bxb.api.add_ons_api import AddOnsApi
from bxb.api.audit_logs_api import AuditLogsApi
from bxb.api.billable_metrics_api import BillableMetricsApi
from bxb.api.billing_entities_api import BillingEntitiesApi
from bxb.api.commitments_api import CommitmentsApi
from bxb.api.coupons_api import CouponsApi
from bxb.api.credit_notes_api import CreditNotesApi
from bxb.api.customers_api import CustomersApi
from bxb.api.dashboard_api import DashboardApi
from bxb.api.data_exports_api import DataExportsApi
from bxb.api.dunning_api import DunningApi
from bxb.api.entitlements_api import EntitlementsApi
from bxb.api.events_api import EventsApi
from bxb.api.features_api import FeaturesApi
from bxb.api.fees_api import FeesApi
from bxb.api.integrations_api import IntegrationsApi
from bxb.api.invoices_api import InvoicesApi
from bxb.api.notifications_api import NotificationsApi
from bxb.api.organizations_api import OrganizationsApi
from bxb.api.payment_methods_api import PaymentMethodsApi
from bxb.api.payment_requests_api import PaymentRequestsApi
from bxb.api.payments_api import PaymentsApi
from bxb.api.plans_api import PlansApi
from bxb.api.portal_api import PortalApi
from bxb.api.search_api import SearchApi
from bxb.api.subscriptions_api import SubscriptionsApi
from bxb.api.taxes_api import TaxesApi
from bxb.api.thresholds_api import ThresholdsApi
from bxb.api.usage_alerts_api import UsageAlertsApi
from bxb.api.wallets_api import WalletsApi
from bxb.api.webhooks_api import WebhooksApi
from bxb.api.default_api import DefaultApi




# coding: utf-8

# flake8: noqa

"""
    bxb API

    A comprehensive usage-based billing platform API. Manage customers, plans, subscriptions, invoices, payments, wallets, coupons, credit notes, taxes, webhooks, and more.

    The version of the OpenAPI document: 0.5.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "0.5.0"

# Define package exports
__all__ = [
    "AddOnsApi",
    "AuditLogsApi",
    "BillableMetricsApi",
    "BillingEntitiesApi",
    "CommitmentsApi",
    "CouponsApi",
    "CreditNotesApi",
    "CustomersApi",
    "DashboardApi",
    "DataExportsApi",
    "DunningApi",
    "EntitlementsApi",
    "EventsApi",
    "FeaturesApi",
    "FeesApi",
    "IntegrationsApi",
    "InvoicesApi",
    "NotificationsApi",
    "OrganizationsApi",
    "PaymentMethodsApi",
    "PaymentRequestsApi",
    "PaymentsApi",
    "PlansApi",
    "PortalApi",
    "SearchApi",
    "SubscriptionsApi",
    "TaxesApi",
    "ThresholdsApi",
    "UsageAlertsApi",
    "WalletsApi",
    "WebhooksApi",
    "DefaultApi",
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "AddOnCreate",
    "AddOnResponse",
    "AddOnUpdate",
    "AggregationType",
    "Amount",
    "Amount1",
    "Amount2",
    "AmountCents",
    "AmountCents1",
    "AmountCents2",
    "AmountCents3",
    "ApiKeyCreate",
    "ApiKeyCreateResponse",
    "ApiKeyListResponse",
    "AppSchemasUsageCurrentUsageResponse",
    "AppSchemasUsageThresholdCurrentUsageResponse",
    "AppliedAddOnDetailResponse",
    "AppliedAddOnResponse",
    "AppliedCouponResponse",
    "AppliedTaxResponse",
    "ApplyAddOnRequest",
    "ApplyCouponRequest",
    "ApplyTaxRequest",
    "AuditLogResponse",
    "BalanceTimelinePoint",
    "BalanceTimelineResponse",
    "BatchPaymentRequestResponse",
    "BatchPaymentRequestResult",
    "BillableMetricCreate",
    "BillableMetricFilterCreate",
    "BillableMetricFilterResponse",
    "BillableMetricResponse",
    "BillableMetricStats",
    "BillableMetricUpdate",
    "BillableMetricUsage",
    "BillingEntityCreate",
    "BillingEntityResponse",
    "BillingEntityUpdate",
    "BillingTime",
    "BulkFinalizeRequest",
    "BulkFinalizeResponse",
    "BulkFinalizeResult",
    "BulkSubscriptionRequest",
    "BulkSubscriptionResponse",
    "BulkSubscriptionResult",
    "BulkTerminateRequest",
    "BulkVoidRequest",
    "BulkVoidResponse",
    "BulkVoidResult",
    "CampaignPreviewInvoiceGroup",
    "CampaignPreviewResponse",
    "CampaignTimelineEvent",
    "CampaignTimelineResponse",
    "ChangePlanPreviewRequest",
    "ChangePlanPreviewResponse",
    "ChargeFilterInput",
    "ChargeInput",
    "ChargeModel",
    "ChargeOutput",
    "ChargeSimulationResult",
    "ChargeUsage",
    "CheckoutSessionCreate",
    "CheckoutSessionResponse",
    "CollectionMetrics",
    "CommitmentCreateAPI",
    "CommitmentResponse",
    "CommitmentUpdate",
    "CouponAnalyticsResponse",
    "CouponCreate",
    "CouponExpiration",
    "CouponFrequency",
    "CouponResponse",
    "CouponStatus",
    "CouponType",
    "CouponUpdate",
    "CreditAmountCents",
    "CreditAmountCents1",
    "CreditNoteCreate",
    "CreditNoteItemCreate",
    "CreditNoteReason",
    "CreditNoteResponse",
    "CreditNoteStatus",
    "CreditNoteType",
    "CreditNoteUpdate",
    "CreditStatus",
    "Credits",
    "CustomerCreate",
    "CustomerHealthResponse",
    "CustomerHealthStatus",
    "CustomerIntegrationMappingResponse",
    "CustomerMetricsResponse",
    "CustomerResponse",
    "CustomerUpdate",
    "DailyRevenuePoint",
    "DashboardStatsResponse",
    "DataExportCreate",
    "DataExportEstimate",
    "DataExportResponse",
    "DepletionForecastResponse",
    "DunningCampaignCreate",
    "DunningCampaignPerformanceStats",
    "DunningCampaignResponse",
    "DunningCampaignThresholdCreate",
    "DunningCampaignThresholdResponse",
    "DunningCampaignUpdate",
    "EndpointDeliveryStats",
    "EntitiesInnerValue",
    "EntitlementCopyRequest",
    "EntitlementCreate",
    "EntitlementResponse",
    "EntitlementUpdate",
    "EstimateFeesRequest",
    "EstimateFeesResponse",
    "EventBatchCreate",
    "EventBatchResponse",
    "EventCreate",
    "EventReprocessResponse",
    "EventResponse",
    "EventVolumePoint",
    "EventVolumeResponse",
    "ExecutionHistoryEntry",
    "ExecutionHistoryInvoice",
    "ExportType",
    "FeatureCreate",
    "FeatureResponse",
    "FeatureType",
    "FeatureUpdate",
    "FeePaymentStatus",
    "FeePreview",
    "FeeResponse",
    "FeeType",
    "FeeUpdate",
    "GlobalSearchResponse",
    "HTTPValidationError",
    "InitialGrantedCredits",
    "IntegrationCreate",
    "IntegrationCustomerResponse",
    "IntegrationMappingResponse",
    "IntegrationResponse",
    "IntegrationSyncHistoryResponse",
    "IntegrationUpdate",
    "InvoiceLineItem",
    "InvoicePreviewRequest",
    "InvoicePreviewResponse",
    "InvoiceResponse",
    "InvoiceSettlementResponse",
    "InvoiceStatus",
    "InvoiceUpdate",
    "LifecycleEvent",
    "LocationInner",
    "ManualPaymentCreate",
    "NetRevenueMetrics",
    "NextBillingDateResponse",
    "NotificationCountResponse",
    "NotificationResponse",
    "OneOffInvoiceCreate",
    "OrganizationCreate",
    "OrganizationCreateResponse",
    "OrganizationResponse",
    "OrganizationUpdate",
    "PaymentAttemptEntry",
    "PaymentAttemptHistoryResponse",
    "PaymentMethodCreate",
    "PaymentMethodResponse",
    "PaymentProvider",
    "PaymentRequestCreate",
    "PaymentRequestInvoiceResponse",
    "PaymentRequestResponse",
    "PaymentResponse",
    "PaymentStatus",
    "PercentageRate",
    "PlanCreate",
    "PlanInterval",
    "PlanResponse",
    "PlanRevenueBreakdown",
    "PlanSimulateRequest",
    "PlanSimulateResponse",
    "PlanSummary",
    "PlanUpdate",
    "PortalAddOnResponse",
    "PortalAppliedCouponResponse",
    "PortalBrandingResponse",
    "PortalChangePlanRequest",
    "PortalDashboardSummaryResponse",
    "PortalNextBillingInfo",
    "PortalPayNowRequest",
    "PortalPayNowResponse",
    "PortalPlanResponse",
    "PortalProfileUpdate",
    "PortalProjectedUsageItem",
    "PortalProjectedUsageResponse",
    "PortalPurchaseAddOnResponse",
    "PortalPurchasedAddOnResponse",
    "PortalQuickActions",
    "PortalRedeemCouponRequest",
    "PortalSubscriptionResponse",
    "PortalTokenResponse",
    "PortalTopUpRequest",
    "PortalTopUpResponse",
    "PortalUpcomingCharge",
    "PortalUrlResponse",
    "PortalUsageLimitItem",
    "PortalUsageLimitsResponse",
    "PortalUsageProgress",
    "PortalUsageTrendPoint",
    "PortalUsageTrendResponse",
    "ProrationDetail",
    "Quantity",
    "Rate",
    "Rate1",
    "RateAmount",
    "RecentActivityResponse",
    "RecentInvoiceItem",
    "RecentSubscriptionItem",
    "RefundAmountCents",
    "RefundAmountCents1",
    "RefundRequest",
    "RefundStatus",
    "RevenueAnalyticsResponse",
    "RevenueByPlanResponse",
    "RevenueByTypeBreakdown",
    "RevenueDataPoint",
    "RevenueResponse",
    "SearchResult",
    "SendReminderResponse",
    "SetupSessionCreate",
    "SetupSessionResponse",
    "SparklineData",
    "SparklinePoint",
    "SubscriptionCreate",
    "SubscriptionLifecycleResponse",
    "SubscriptionMetricsResponse",
    "SubscriptionPlanBreakdown",
    "SubscriptionResponse",
    "SubscriptionStatus",
    "SubscriptionUpdate",
    "TaxApplicationCountsResponse",
    "TaxAppliedEntitiesResponse",
    "TaxCreate",
    "TaxResponse",
    "TaxUpdate",
    "TaxesAmountCents",
    "TaxesAmountCents1",
    "TerminationAction",
    "ThresholdValue",
    "ThresholdValue1",
    "TopCustomerRevenue",
    "TotalAmountCents",
    "TotalAmountCents1",
    "TrendIndicator",
    "UnitPrice",
    "UsageAlertCreate",
    "UsageAlertResponse",
    "UsageAlertStatusResponse",
    "UsageAlertTriggerResponse",
    "UsageAlertUpdate",
    "UsageMetricVolume",
    "UsageMetricsResponse",
    "UsageThresholdCreateAPI",
    "UsageThresholdResponse",
    "UsageTrendPoint",
    "UsageTrendResponse",
    "ValidationError",
    "WalletCreate",
    "WalletResponse",
    "WalletStatus",
    "WalletTopUp",
    "WalletTransactionResponse",
    "WalletTransferRequest",
    "WalletTransferResponse",
    "WalletUpdate",
    "WebhookDeliveryAttemptResponse",
    "WebhookEndpointCreate",
    "WebhookEndpointResponse",
    "WebhookEndpointUpdate",
    "WebhookResponse",
]

# import apis into sdk package
from bxb.api.add_ons_api import AddOnsApi as AddOnsApi
from bxb.api.audit_logs_api import AuditLogsApi as AuditLogsApi
from bxb.api.billable_metrics_api import BillableMetricsApi as BillableMetricsApi
from bxb.api.billing_entities_api import BillingEntitiesApi as BillingEntitiesApi
from bxb.api.commitments_api import CommitmentsApi as CommitmentsApi
from bxb.api.coupons_api import CouponsApi as CouponsApi
from bxb.api.credit_notes_api import CreditNotesApi as CreditNotesApi
from bxb.api.customers_api import CustomersApi as CustomersApi
from bxb.api.dashboard_api import DashboardApi as DashboardApi
from bxb.api.data_exports_api import DataExportsApi as DataExportsApi
from bxb.api.dunning_api import DunningApi as DunningApi
from bxb.api.entitlements_api import EntitlementsApi as EntitlementsApi
from bxb.api.events_api import EventsApi as EventsApi
from bxb.api.features_api import FeaturesApi as FeaturesApi
from bxb.api.fees_api import FeesApi as FeesApi
from bxb.api.integrations_api import IntegrationsApi as IntegrationsApi
from bxb.api.invoices_api import InvoicesApi as InvoicesApi
from bxb.api.notifications_api import NotificationsApi as NotificationsApi
from bxb.api.organizations_api import OrganizationsApi as OrganizationsApi
from bxb.api.payment_methods_api import PaymentMethodsApi as PaymentMethodsApi
from bxb.api.payment_requests_api import PaymentRequestsApi as PaymentRequestsApi
from bxb.api.payments_api import PaymentsApi as PaymentsApi
from bxb.api.plans_api import PlansApi as PlansApi
from bxb.api.portal_api import PortalApi as PortalApi
from bxb.api.search_api import SearchApi as SearchApi
from bxb.api.subscriptions_api import SubscriptionsApi as SubscriptionsApi
from bxb.api.taxes_api import TaxesApi as TaxesApi
from bxb.api.thresholds_api import ThresholdsApi as ThresholdsApi
from bxb.api.usage_alerts_api import UsageAlertsApi as UsageAlertsApi
from bxb.api.wallets_api import WalletsApi as WalletsApi
from bxb.api.webhooks_api import WebhooksApi as WebhooksApi
from bxb.api.default_api import DefaultApi as DefaultApi

# import ApiClient
from bxb.api_response import ApiResponse as ApiResponse
from bxb.api_client import ApiClient as ApiClient
from bxb.configuration import Configuration as Configuration
from bxb.exceptions import OpenApiException as OpenApiException
from bxb.exceptions import ApiTypeError as ApiTypeError
from bxb.exceptions import ApiValueError as ApiValueError
from bxb.exceptions import ApiKeyError as ApiKeyError
from bxb.exceptions import ApiAttributeError as ApiAttributeError
from bxb.exceptions import ApiException as ApiException

# import models into sdk package
from bxb.models.add_on_create import AddOnCreate as AddOnCreate
from bxb.models.add_on_response import AddOnResponse as AddOnResponse
from bxb.models.add_on_update import AddOnUpdate as AddOnUpdate
from bxb.models.aggregation_type import AggregationType as AggregationType
from bxb.models.amount import Amount as Amount
from bxb.models.amount1 import Amount1 as Amount1
from bxb.models.amount2 import Amount2 as Amount2
from bxb.models.amount_cents import AmountCents as AmountCents
from bxb.models.amount_cents1 import AmountCents1 as AmountCents1
from bxb.models.amount_cents2 import AmountCents2 as AmountCents2
from bxb.models.amount_cents3 import AmountCents3 as AmountCents3
from bxb.models.api_key_create import ApiKeyCreate as ApiKeyCreate
from bxb.models.api_key_create_response import ApiKeyCreateResponse as ApiKeyCreateResponse
from bxb.models.api_key_list_response import ApiKeyListResponse as ApiKeyListResponse
from bxb.models.app_schemas_usage_current_usage_response import AppSchemasUsageCurrentUsageResponse as AppSchemasUsageCurrentUsageResponse
from bxb.models.app_schemas_usage_threshold_current_usage_response import AppSchemasUsageThresholdCurrentUsageResponse as AppSchemasUsageThresholdCurrentUsageResponse
from bxb.models.applied_add_on_detail_response import AppliedAddOnDetailResponse as AppliedAddOnDetailResponse
from bxb.models.applied_add_on_response import AppliedAddOnResponse as AppliedAddOnResponse
from bxb.models.applied_coupon_response import AppliedCouponResponse as AppliedCouponResponse
from bxb.models.applied_tax_response import AppliedTaxResponse as AppliedTaxResponse
from bxb.models.apply_add_on_request import ApplyAddOnRequest as ApplyAddOnRequest
from bxb.models.apply_coupon_request import ApplyCouponRequest as ApplyCouponRequest
from bxb.models.apply_tax_request import ApplyTaxRequest as ApplyTaxRequest
from bxb.models.audit_log_response import AuditLogResponse as AuditLogResponse
from bxb.models.balance_timeline_point import BalanceTimelinePoint as BalanceTimelinePoint
from bxb.models.balance_timeline_response import BalanceTimelineResponse as BalanceTimelineResponse
from bxb.models.batch_payment_request_response import BatchPaymentRequestResponse as BatchPaymentRequestResponse
from bxb.models.batch_payment_request_result import BatchPaymentRequestResult as BatchPaymentRequestResult
from bxb.models.billable_metric_create import BillableMetricCreate as BillableMetricCreate
from bxb.models.billable_metric_filter_create import BillableMetricFilterCreate as BillableMetricFilterCreate
from bxb.models.billable_metric_filter_response import BillableMetricFilterResponse as BillableMetricFilterResponse
from bxb.models.billable_metric_response import BillableMetricResponse as BillableMetricResponse
from bxb.models.billable_metric_stats import BillableMetricStats as BillableMetricStats
from bxb.models.billable_metric_update import BillableMetricUpdate as BillableMetricUpdate
from bxb.models.billable_metric_usage import BillableMetricUsage as BillableMetricUsage
from bxb.models.billing_entity_create import BillingEntityCreate as BillingEntityCreate
from bxb.models.billing_entity_response import BillingEntityResponse as BillingEntityResponse
from bxb.models.billing_entity_update import BillingEntityUpdate as BillingEntityUpdate
from bxb.models.billing_time import BillingTime as BillingTime
from bxb.models.bulk_finalize_request import BulkFinalizeRequest as BulkFinalizeRequest
from bxb.models.bulk_finalize_response import BulkFinalizeResponse as BulkFinalizeResponse
from bxb.models.bulk_finalize_result import BulkFinalizeResult as BulkFinalizeResult
from bxb.models.bulk_subscription_request import BulkSubscriptionRequest as BulkSubscriptionRequest
from bxb.models.bulk_subscription_response import BulkSubscriptionResponse as BulkSubscriptionResponse
from bxb.models.bulk_subscription_result import BulkSubscriptionResult as BulkSubscriptionResult
from bxb.models.bulk_terminate_request import BulkTerminateRequest as BulkTerminateRequest
from bxb.models.bulk_void_request import BulkVoidRequest as BulkVoidRequest
from bxb.models.bulk_void_response import BulkVoidResponse as BulkVoidResponse
from bxb.models.bulk_void_result import BulkVoidResult as BulkVoidResult
from bxb.models.campaign_preview_invoice_group import CampaignPreviewInvoiceGroup as CampaignPreviewInvoiceGroup
from bxb.models.campaign_preview_response import CampaignPreviewResponse as CampaignPreviewResponse
from bxb.models.campaign_timeline_event import CampaignTimelineEvent as CampaignTimelineEvent
from bxb.models.campaign_timeline_response import CampaignTimelineResponse as CampaignTimelineResponse
from bxb.models.change_plan_preview_request import ChangePlanPreviewRequest as ChangePlanPreviewRequest
from bxb.models.change_plan_preview_response import ChangePlanPreviewResponse as ChangePlanPreviewResponse
from bxb.models.charge_filter_input import ChargeFilterInput as ChargeFilterInput
from bxb.models.charge_input import ChargeInput as ChargeInput
from bxb.models.charge_model import ChargeModel as ChargeModel
from bxb.models.charge_output import ChargeOutput as ChargeOutput
from bxb.models.charge_simulation_result import ChargeSimulationResult as ChargeSimulationResult
from bxb.models.charge_usage import ChargeUsage as ChargeUsage
from bxb.models.checkout_session_create import CheckoutSessionCreate as CheckoutSessionCreate
from bxb.models.checkout_session_response import CheckoutSessionResponse as CheckoutSessionResponse
from bxb.models.collection_metrics import CollectionMetrics as CollectionMetrics
from bxb.models.commitment_create_api import CommitmentCreateAPI as CommitmentCreateAPI
from bxb.models.commitment_response import CommitmentResponse as CommitmentResponse
from bxb.models.commitment_update import CommitmentUpdate as CommitmentUpdate
from bxb.models.coupon_analytics_response import CouponAnalyticsResponse as CouponAnalyticsResponse
from bxb.models.coupon_create import CouponCreate as CouponCreate
from bxb.models.coupon_expiration import CouponExpiration as CouponExpiration
from bxb.models.coupon_frequency import CouponFrequency as CouponFrequency
from bxb.models.coupon_response import CouponResponse as CouponResponse
from bxb.models.coupon_status import CouponStatus as CouponStatus
from bxb.models.coupon_type import CouponType as CouponType
from bxb.models.coupon_update import CouponUpdate as CouponUpdate
from bxb.models.credit_amount_cents import CreditAmountCents as CreditAmountCents
from bxb.models.credit_amount_cents1 import CreditAmountCents1 as CreditAmountCents1
from bxb.models.credit_note_create import CreditNoteCreate as CreditNoteCreate
from bxb.models.credit_note_item_create import CreditNoteItemCreate as CreditNoteItemCreate
from bxb.models.credit_note_reason import CreditNoteReason as CreditNoteReason
from bxb.models.credit_note_response import CreditNoteResponse as CreditNoteResponse
from bxb.models.credit_note_status import CreditNoteStatus as CreditNoteStatus
from bxb.models.credit_note_type import CreditNoteType as CreditNoteType
from bxb.models.credit_note_update import CreditNoteUpdate as CreditNoteUpdate
from bxb.models.credit_status import CreditStatus as CreditStatus
from bxb.models.credits import Credits as Credits
from bxb.models.customer_create import CustomerCreate as CustomerCreate
from bxb.models.customer_health_response import CustomerHealthResponse as CustomerHealthResponse
from bxb.models.customer_health_status import CustomerHealthStatus as CustomerHealthStatus
from bxb.models.customer_integration_mapping_response import CustomerIntegrationMappingResponse as CustomerIntegrationMappingResponse
from bxb.models.customer_metrics_response import CustomerMetricsResponse as CustomerMetricsResponse
from bxb.models.customer_response import CustomerResponse as CustomerResponse
from bxb.models.customer_update import CustomerUpdate as CustomerUpdate
from bxb.models.daily_revenue_point import DailyRevenuePoint as DailyRevenuePoint
from bxb.models.dashboard_stats_response import DashboardStatsResponse as DashboardStatsResponse
from bxb.models.data_export_create import DataExportCreate as DataExportCreate
from bxb.models.data_export_estimate import DataExportEstimate as DataExportEstimate
from bxb.models.data_export_response import DataExportResponse as DataExportResponse
from bxb.models.depletion_forecast_response import DepletionForecastResponse as DepletionForecastResponse
from bxb.models.dunning_campaign_create import DunningCampaignCreate as DunningCampaignCreate
from bxb.models.dunning_campaign_performance_stats import DunningCampaignPerformanceStats as DunningCampaignPerformanceStats
from bxb.models.dunning_campaign_response import DunningCampaignResponse as DunningCampaignResponse
from bxb.models.dunning_campaign_threshold_create import DunningCampaignThresholdCreate as DunningCampaignThresholdCreate
from bxb.models.dunning_campaign_threshold_response import DunningCampaignThresholdResponse as DunningCampaignThresholdResponse
from bxb.models.dunning_campaign_update import DunningCampaignUpdate as DunningCampaignUpdate
from bxb.models.endpoint_delivery_stats import EndpointDeliveryStats as EndpointDeliveryStats
from bxb.models.entities_inner_value import EntitiesInnerValue as EntitiesInnerValue
from bxb.models.entitlement_copy_request import EntitlementCopyRequest as EntitlementCopyRequest
from bxb.models.entitlement_create import EntitlementCreate as EntitlementCreate
from bxb.models.entitlement_response import EntitlementResponse as EntitlementResponse
from bxb.models.entitlement_update import EntitlementUpdate as EntitlementUpdate
from bxb.models.estimate_fees_request import EstimateFeesRequest as EstimateFeesRequest
from bxb.models.estimate_fees_response import EstimateFeesResponse as EstimateFeesResponse
from bxb.models.event_batch_create import EventBatchCreate as EventBatchCreate
from bxb.models.event_batch_response import EventBatchResponse as EventBatchResponse
from bxb.models.event_create import EventCreate as EventCreate
from bxb.models.event_reprocess_response import EventReprocessResponse as EventReprocessResponse
from bxb.models.event_response import EventResponse as EventResponse
from bxb.models.event_volume_point import EventVolumePoint as EventVolumePoint
from bxb.models.event_volume_response import EventVolumeResponse as EventVolumeResponse
from bxb.models.execution_history_entry import ExecutionHistoryEntry as ExecutionHistoryEntry
from bxb.models.execution_history_invoice import ExecutionHistoryInvoice as ExecutionHistoryInvoice
from bxb.models.export_type import ExportType as ExportType
from bxb.models.feature_create import FeatureCreate as FeatureCreate
from bxb.models.feature_response import FeatureResponse as FeatureResponse
from bxb.models.feature_type import FeatureType as FeatureType
from bxb.models.feature_update import FeatureUpdate as FeatureUpdate
from bxb.models.fee_payment_status import FeePaymentStatus as FeePaymentStatus
from bxb.models.fee_preview import FeePreview as FeePreview
from bxb.models.fee_response import FeeResponse as FeeResponse
from bxb.models.fee_type import FeeType as FeeType
from bxb.models.fee_update import FeeUpdate as FeeUpdate
from bxb.models.global_search_response import GlobalSearchResponse as GlobalSearchResponse
from bxb.models.http_validation_error import HTTPValidationError as HTTPValidationError
from bxb.models.initial_granted_credits import InitialGrantedCredits as InitialGrantedCredits
from bxb.models.integration_create import IntegrationCreate as IntegrationCreate
from bxb.models.integration_customer_response import IntegrationCustomerResponse as IntegrationCustomerResponse
from bxb.models.integration_mapping_response import IntegrationMappingResponse as IntegrationMappingResponse
from bxb.models.integration_response import IntegrationResponse as IntegrationResponse
from bxb.models.integration_sync_history_response import IntegrationSyncHistoryResponse as IntegrationSyncHistoryResponse
from bxb.models.integration_update import IntegrationUpdate as IntegrationUpdate
from bxb.models.invoice_line_item import InvoiceLineItem as InvoiceLineItem
from bxb.models.invoice_preview_request import InvoicePreviewRequest as InvoicePreviewRequest
from bxb.models.invoice_preview_response import InvoicePreviewResponse as InvoicePreviewResponse
from bxb.models.invoice_response import InvoiceResponse as InvoiceResponse
from bxb.models.invoice_settlement_response import InvoiceSettlementResponse as InvoiceSettlementResponse
from bxb.models.invoice_status import InvoiceStatus as InvoiceStatus
from bxb.models.invoice_update import InvoiceUpdate as InvoiceUpdate
from bxb.models.lifecycle_event import LifecycleEvent as LifecycleEvent
from bxb.models.location_inner import LocationInner as LocationInner
from bxb.models.manual_payment_create import ManualPaymentCreate as ManualPaymentCreate
from bxb.models.net_revenue_metrics import NetRevenueMetrics as NetRevenueMetrics
from bxb.models.next_billing_date_response import NextBillingDateResponse as NextBillingDateResponse
from bxb.models.notification_count_response import NotificationCountResponse as NotificationCountResponse
from bxb.models.notification_response import NotificationResponse as NotificationResponse
from bxb.models.one_off_invoice_create import OneOffInvoiceCreate as OneOffInvoiceCreate
from bxb.models.organization_create import OrganizationCreate as OrganizationCreate
from bxb.models.organization_create_response import OrganizationCreateResponse as OrganizationCreateResponse
from bxb.models.organization_response import OrganizationResponse as OrganizationResponse
from bxb.models.organization_update import OrganizationUpdate as OrganizationUpdate
from bxb.models.payment_attempt_entry import PaymentAttemptEntry as PaymentAttemptEntry
from bxb.models.payment_attempt_history_response import PaymentAttemptHistoryResponse as PaymentAttemptHistoryResponse
from bxb.models.payment_method_create import PaymentMethodCreate as PaymentMethodCreate
from bxb.models.payment_method_response import PaymentMethodResponse as PaymentMethodResponse
from bxb.models.payment_provider import PaymentProvider as PaymentProvider
from bxb.models.payment_request_create import PaymentRequestCreate as PaymentRequestCreate
from bxb.models.payment_request_invoice_response import PaymentRequestInvoiceResponse as PaymentRequestInvoiceResponse
from bxb.models.payment_request_response import PaymentRequestResponse as PaymentRequestResponse
from bxb.models.payment_response import PaymentResponse as PaymentResponse
from bxb.models.payment_status import PaymentStatus as PaymentStatus
from bxb.models.percentage_rate import PercentageRate as PercentageRate
from bxb.models.plan_create import PlanCreate as PlanCreate
from bxb.models.plan_interval import PlanInterval as PlanInterval
from bxb.models.plan_response import PlanResponse as PlanResponse
from bxb.models.plan_revenue_breakdown import PlanRevenueBreakdown as PlanRevenueBreakdown
from bxb.models.plan_simulate_request import PlanSimulateRequest as PlanSimulateRequest
from bxb.models.plan_simulate_response import PlanSimulateResponse as PlanSimulateResponse
from bxb.models.plan_summary import PlanSummary as PlanSummary
from bxb.models.plan_update import PlanUpdate as PlanUpdate
from bxb.models.portal_add_on_response import PortalAddOnResponse as PortalAddOnResponse
from bxb.models.portal_applied_coupon_response import PortalAppliedCouponResponse as PortalAppliedCouponResponse
from bxb.models.portal_branding_response import PortalBrandingResponse as PortalBrandingResponse
from bxb.models.portal_change_plan_request import PortalChangePlanRequest as PortalChangePlanRequest
from bxb.models.portal_dashboard_summary_response import PortalDashboardSummaryResponse as PortalDashboardSummaryResponse
from bxb.models.portal_next_billing_info import PortalNextBillingInfo as PortalNextBillingInfo
from bxb.models.portal_pay_now_request import PortalPayNowRequest as PortalPayNowRequest
from bxb.models.portal_pay_now_response import PortalPayNowResponse as PortalPayNowResponse
from bxb.models.portal_plan_response import PortalPlanResponse as PortalPlanResponse
from bxb.models.portal_profile_update import PortalProfileUpdate as PortalProfileUpdate
from bxb.models.portal_projected_usage_item import PortalProjectedUsageItem as PortalProjectedUsageItem
from bxb.models.portal_projected_usage_response import PortalProjectedUsageResponse as PortalProjectedUsageResponse
from bxb.models.portal_purchase_add_on_response import PortalPurchaseAddOnResponse as PortalPurchaseAddOnResponse
from bxb.models.portal_purchased_add_on_response import PortalPurchasedAddOnResponse as PortalPurchasedAddOnResponse
from bxb.models.portal_quick_actions import PortalQuickActions as PortalQuickActions
from bxb.models.portal_redeem_coupon_request import PortalRedeemCouponRequest as PortalRedeemCouponRequest
from bxb.models.portal_subscription_response import PortalSubscriptionResponse as PortalSubscriptionResponse
from bxb.models.portal_token_response import PortalTokenResponse as PortalTokenResponse
from bxb.models.portal_top_up_request import PortalTopUpRequest as PortalTopUpRequest
from bxb.models.portal_top_up_response import PortalTopUpResponse as PortalTopUpResponse
from bxb.models.portal_upcoming_charge import PortalUpcomingCharge as PortalUpcomingCharge
from bxb.models.portal_url_response import PortalUrlResponse as PortalUrlResponse
from bxb.models.portal_usage_limit_item import PortalUsageLimitItem as PortalUsageLimitItem
from bxb.models.portal_usage_limits_response import PortalUsageLimitsResponse as PortalUsageLimitsResponse
from bxb.models.portal_usage_progress import PortalUsageProgress as PortalUsageProgress
from bxb.models.portal_usage_trend_point import PortalUsageTrendPoint as PortalUsageTrendPoint
from bxb.models.portal_usage_trend_response import PortalUsageTrendResponse as PortalUsageTrendResponse
from bxb.models.proration_detail import ProrationDetail as ProrationDetail
from bxb.models.quantity import Quantity as Quantity
from bxb.models.rate import Rate as Rate
from bxb.models.rate1 import Rate1 as Rate1
from bxb.models.rate_amount import RateAmount as RateAmount
from bxb.models.recent_activity_response import RecentActivityResponse as RecentActivityResponse
from bxb.models.recent_invoice_item import RecentInvoiceItem as RecentInvoiceItem
from bxb.models.recent_subscription_item import RecentSubscriptionItem as RecentSubscriptionItem
from bxb.models.refund_amount_cents import RefundAmountCents as RefundAmountCents
from bxb.models.refund_amount_cents1 import RefundAmountCents1 as RefundAmountCents1
from bxb.models.refund_request import RefundRequest as RefundRequest
from bxb.models.refund_status import RefundStatus as RefundStatus
from bxb.models.revenue_analytics_response import RevenueAnalyticsResponse as RevenueAnalyticsResponse
from bxb.models.revenue_by_plan_response import RevenueByPlanResponse as RevenueByPlanResponse
from bxb.models.revenue_by_type_breakdown import RevenueByTypeBreakdown as RevenueByTypeBreakdown
from bxb.models.revenue_data_point import RevenueDataPoint as RevenueDataPoint
from bxb.models.revenue_response import RevenueResponse as RevenueResponse
from bxb.models.search_result import SearchResult as SearchResult
from bxb.models.send_reminder_response import SendReminderResponse as SendReminderResponse
from bxb.models.setup_session_create import SetupSessionCreate as SetupSessionCreate
from bxb.models.setup_session_response import SetupSessionResponse as SetupSessionResponse
from bxb.models.sparkline_data import SparklineData as SparklineData
from bxb.models.sparkline_point import SparklinePoint as SparklinePoint
from bxb.models.subscription_create import SubscriptionCreate as SubscriptionCreate
from bxb.models.subscription_lifecycle_response import SubscriptionLifecycleResponse as SubscriptionLifecycleResponse
from bxb.models.subscription_metrics_response import SubscriptionMetricsResponse as SubscriptionMetricsResponse
from bxb.models.subscription_plan_breakdown import SubscriptionPlanBreakdown as SubscriptionPlanBreakdown
from bxb.models.subscription_response import SubscriptionResponse as SubscriptionResponse
from bxb.models.subscription_status import SubscriptionStatus as SubscriptionStatus
from bxb.models.subscription_update import SubscriptionUpdate as SubscriptionUpdate
from bxb.models.tax_application_counts_response import TaxApplicationCountsResponse as TaxApplicationCountsResponse
from bxb.models.tax_applied_entities_response import TaxAppliedEntitiesResponse as TaxAppliedEntitiesResponse
from bxb.models.tax_create import TaxCreate as TaxCreate
from bxb.models.tax_response import TaxResponse as TaxResponse
from bxb.models.tax_update import TaxUpdate as TaxUpdate
from bxb.models.taxes_amount_cents import TaxesAmountCents as TaxesAmountCents
from bxb.models.taxes_amount_cents1 import TaxesAmountCents1 as TaxesAmountCents1
from bxb.models.termination_action import TerminationAction as TerminationAction
from bxb.models.threshold_value import ThresholdValue as ThresholdValue
from bxb.models.threshold_value1 import ThresholdValue1 as ThresholdValue1
from bxb.models.top_customer_revenue import TopCustomerRevenue as TopCustomerRevenue
from bxb.models.total_amount_cents import TotalAmountCents as TotalAmountCents
from bxb.models.total_amount_cents1 import TotalAmountCents1 as TotalAmountCents1
from bxb.models.trend_indicator import TrendIndicator as TrendIndicator
from bxb.models.unit_price import UnitPrice as UnitPrice
from bxb.models.usage_alert_create import UsageAlertCreate as UsageAlertCreate
from bxb.models.usage_alert_response import UsageAlertResponse as UsageAlertResponse
from bxb.models.usage_alert_status_response import UsageAlertStatusResponse as UsageAlertStatusResponse
from bxb.models.usage_alert_trigger_response import UsageAlertTriggerResponse as UsageAlertTriggerResponse
from bxb.models.usage_alert_update import UsageAlertUpdate as UsageAlertUpdate
from bxb.models.usage_metric_volume import UsageMetricVolume as UsageMetricVolume
from bxb.models.usage_metrics_response import UsageMetricsResponse as UsageMetricsResponse
from bxb.models.usage_threshold_create_api import UsageThresholdCreateAPI as UsageThresholdCreateAPI
from bxb.models.usage_threshold_response import UsageThresholdResponse as UsageThresholdResponse
from bxb.models.usage_trend_point import UsageTrendPoint as UsageTrendPoint
from bxb.models.usage_trend_response import UsageTrendResponse as UsageTrendResponse
from bxb.models.validation_error import ValidationError as ValidationError
from bxb.models.wallet_create import WalletCreate as WalletCreate
from bxb.models.wallet_response import WalletResponse as WalletResponse
from bxb.models.wallet_status import WalletStatus as WalletStatus
from bxb.models.wallet_top_up import WalletTopUp as WalletTopUp
from bxb.models.wallet_transaction_response import WalletTransactionResponse as WalletTransactionResponse
from bxb.models.wallet_transfer_request import WalletTransferRequest as WalletTransferRequest
from bxb.models.wallet_transfer_response import WalletTransferResponse as WalletTransferResponse
from bxb.models.wallet_update import WalletUpdate as WalletUpdate
from bxb.models.webhook_delivery_attempt_response import WebhookDeliveryAttemptResponse as WebhookDeliveryAttemptResponse
from bxb.models.webhook_endpoint_create import WebhookEndpointCreate as WebhookEndpointCreate
from bxb.models.webhook_endpoint_response import WebhookEndpointResponse as WebhookEndpointResponse
from bxb.models.webhook_endpoint_update import WebhookEndpointUpdate as WebhookEndpointUpdate
from bxb.models.webhook_response import WebhookResponse as WebhookResponse

